@@ -258,99 +258,113 @@ def json_logic_replace_data(rule, data, string_data=None, json_data=None):
         return json_data
 
 
-def run_external_workflow(conf, external_workflow_config, vtiger_access):
+def run_external_workflow(conf, task_id, vtiger_access):
     """ this function will get check if conditions are satisfied for triggering external workflow or not.
         input:  1.conf : conf object
-                2.external_workflow_config : config dictionary containing workflow, event_type, search_object
-        1. get configs and create query.
-        2. replace data using json logic.
-        3. call trigger workflow function if conditions are satisfied.
+                2.task_id: current task id
+                3.vtiger_access: vtiger_access config dict
+
+        1. get external_workflow_config : config dictionary containing workflow, event_type, search_object
+        2. get configs and create query.
+        3. replace data using json logic.
+        4. call trigger workflow function if conditions are satisfied.
         """
 
-    data = conf.get('data', '')
-    workflow = external_workflow_config.get('workflow', '')
-    event_type = external_workflow_config.get('event_type', '')
+    if conf.get('external_workflow_config', ''):
 
-    search_object = external_workflow_config.get('search_object', '')
-    print(f"Data: {data} \n Workflow: {workflow} \n Event Type: {event_type} \n Search Object: {search_object}")
-    if search_object and search_object.get('condition_object', ''):
-        rule = search_object.get('rule', '')
+        if task_id in conf.get('external_workflow_config', ''):
+            external_workflow_config = conf.get('external_workflow_config').get(task_id, '')
 
-        # prepare query
-        condition = buildquery(search_object.get('condition_object'))
-        module = search_object.get('search_module').get('name')
-        limit = str(search_object.get('fetch_record', ''))
+        if external_workflow_config:
 
-        if event_type != 'import' and event_type != 'manual':
-            limit = '1'
+            data = conf.get('data', '')
+            workflow = external_workflow_config.get('workflow', '')
+            event_type = external_workflow_config.get('event_type', '')
 
-        order_by = ''
-        if search_object.get('sort', ''):
-            order_by = search_object.get('sort').get('column') + " " + search_object.get('sort').get(
-                'type', "")
+            search_object = external_workflow_config.get('search_object', '')
+            print(f"Data: {data} \n Workflow: {workflow} \n Event Type: {event_type} \n Search Object: {search_object}")
+            if search_object and search_object.get('condition_object', ''):
+                rule = search_object.get('rule', '')
 
-        if limit and order_by:
-            query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
-                module=module,
-                condition=condition,
-                order_by=order_by,
-                limit=limit)
-        elif limit:
-            query = 'SELECT * FROM {module} WHERE {condition} LIMIT {limit};'.format(
-                module=module,
-                condition=condition,
-                limit=limit)
-        elif order_by:
-            query = 'SELECT * FROM {module} WHERE {condition} order by {order_by};'.format(
-                module=module,
-                condition=condition,
-                order_by=order_by,
-            )
-        else:
-            query = 'SELECT * FROM {module} WHERE {condition};'.format(
-                module=module,
-                condition=condition,
-            )
+                # prepare query
+                condition = buildquery(search_object.get('condition_object'))
+                module = search_object.get('search_module').get('name')
+                limit = str(search_object.get('fetch_record', ''))
 
-        # replace variable name with data using JSON_LOGIC.
-        print(f"Query: {query}")
-        if rule:
-            query = json_logic_replace_data(rule, conf, query)
+                if event_type != 'import' and event_type != 'manual':
+                    limit = '1'
 
-        session_name = vtiger_access.get('vtiger_session', '')
-        print("Session name: {}".format(session_name))
-        if not session_name:
-            print("unable to get session id from vtiger access. Please try again")
-            return None
+                order_by = ''
+                if search_object.get('sort', ''):
+                    order_by = search_object.get('sort').get('column') + " " + search_object.get('sort').get(
+                        'type', "")
 
-        url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
-            sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
-        print(f"Url: {url}")
-        headers = {'content-type': 'application/json'}
-        request_type = 'GET'
-        response, status = invoke_http_request(url, request_type, headers)
-
-        print(f"Response: {response}")
-
-        if response:
-            # trigger only if condition is satisfied
-            if response.get('result') and workflow and event_type:
-
-                if event_type == 'import' or event_type == 'manual':
-                    # if event type = import/ manual then pass list
-
-                    trigger_workflow(workflow, event_type, response.get('result'),
-                                     vtiger_access.get('service_url'))
+                if limit and order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                        limit=limit)
+                elif limit:
+                    query = 'SELECT * FROM {module} WHERE {condition} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        limit=limit)
+                elif order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                    )
                 else:
-                    trigger_workflow(workflow, event_type, response.get('result')[0],
-                                     vtiger_access.get('service_url'))
-    elif workflow and event_type and data:
-        # trigger without condition
-        if event_type == 'import' or event_type == 'manual':
-            list_of_data = [data]
-            trigger_workflow(workflow, event_type, list_of_data, vtiger_access.get('service_url'))
+                    query = 'SELECT * FROM {module} WHERE {condition};'.format(
+                        module=module,
+                        condition=condition,
+                    )
+
+                # replace variable name with data using JSON_LOGIC.
+                print(f"Query: {query}")
+                if rule:
+                    query = json_logic_replace_data(rule, conf, query)
+
+                session_name = vtiger_access.get('vtiger_session', '')
+                print("Session name: {}".format(session_name))
+                if not session_name:
+                    print("unable to get session id from vtiger access. Please try again")
+                    return None
+
+                url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
+                    sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
+                print(f"Url: {url}")
+                headers = {'content-type': 'application/json'}
+                request_type = 'GET'
+                response, status = invoke_http_request(url, request_type, headers)
+
+                print(f"Response: {response}")
+
+                if response:
+                    # trigger only if condition is satisfied
+                    if response.get('result') and workflow and event_type:
+
+                        if event_type == 'import' or event_type == 'manual':
+                            # if event type = import/ manual then pass list
+
+                            trigger_workflow(workflow, event_type, response.get('result'),
+                                             vtiger_access.get('service_url'))
+                        else:
+                            trigger_workflow(workflow, event_type, response.get('result')[0],
+                                             vtiger_access.get('service_url'))
+            elif workflow and event_type and data:
+                # trigger without condition
+                if event_type == 'import' or event_type == 'manual':
+                    list_of_data = [data]
+                    trigger_workflow(workflow, event_type, list_of_data, vtiger_access.get('service_url'))
+                else:
+                    trigger_workflow(workflow, event_type, data, vtiger_access.get('service_url'))
         else:
-            trigger_workflow(workflow, event_type, data, vtiger_access.get('service_url'))
+            print("external_workflow_config not found for task id:", task_id)
+    else:
+        print("external_workflow_config not found in conf")
 
 
 def trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_access):
@@ -415,145 +429,223 @@ def trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_ac
             return None
 
 
-def invoke_set_value_task(conf, set_value_configs, vtiger_access):
+def invoke_set_value_task(conf, task_id, vtiger_access):
     """   this function will execute set value task.
             params:
             1.conf : conf object
-            2.set_value_configs : conf dictionary containing rule, search_module_object, set_value_fields
+            2.task_id: current task id
             3.vtiger_access : json object containing vtiger credentials
 
             Execution:
-            1. get configs and create query and replace data using json logic.
-            2. check if conditions are satisfied using get query.
-            3. call trigger_set_value_task function if conditions are satisfied.
+            1. get set_value_configs : conf dictionary containing rule, search_module_object, set_value_fields
+            2. get configs and create query and replace data using json logic.
+            3. check if conditions are satisfied using get query.
+            4. call trigger_set_value_task function if conditions are satisfied.
             """
 
     print("Set value task invoked")
-    set_value_fields = set_value_configs.get('set_value_fields', '')
-    search_module_object = set_value_configs.get('search_module_object', '')
-    rule = set_value_configs.get('rule', '')
+    if conf.get('set_value_configs', ''):
 
-    session_name = vtiger_access.get('vtiger_session', '')
-    print("Session name: {}".format(session_name))
-    if not session_name:
-        print("unable to get session id from vtiger access. Please try again")
-        return None
+        if task_id in conf.get('set_value_configs', ''):
+            set_value_configs = conf.get('set_value_configs').get(task_id, '')
 
-    print("Search module object: {}".format(search_module_object))
-    print("Set value fields: {}".format(set_value_fields))
-    if search_module_object and search_module_object.get('condition_object', ''):
-        condition = buildquery(search_module_object.get('condition_object'))
+        if set_value_configs:
+            set_value_fields = set_value_configs.get('set_value_fields', '')
+            search_module_object = set_value_configs.get('search_module_object', '')
+            rule = set_value_configs.get('rule', '')
 
-        print("Condition {}".format(condition))
+            session_name = vtiger_access.get('vtiger_session', '')
+            print("Session name: {}".format(session_name))
+            if not session_name:
+                print("unable to get session id from vtiger access. Please try again")
+                return None
 
-        module = search_module_object.get('name')
+            print("Search module object: {}".format(search_module_object))
+            print("Set value fields: {}".format(set_value_fields))
+            if search_module_object and search_module_object.get('condition_object', ''):
+                condition = buildquery(search_module_object.get('condition_object'))
 
-        limit = str(search_module_object.get('fetch_record', ''))
+                print("Condition {}".format(condition))
 
-        # prepare query using search_module_object
+                module = search_module_object.get('name')
 
-        order_by = ''
-        if search_module_object.get('sort', ''):
-            order_by = search_module_object.get('sort').get('column') + " " + search_module_object.get('sort').get(
-                'type', "")
+                limit = str(search_module_object.get('fetch_record', ''))
 
-        if limit and order_by:
-            query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
-                module=module,
-                condition=condition,
-                order_by=order_by,
-                limit=limit)
-        elif limit:
-            query = 'SELECT * FROM {module} WHERE {condition} LIMIT {limit};'.format(
-                module=module,
-                condition=condition,
-                limit=limit)
-        elif order_by:
-            query = 'SELECT * FROM {module} WHERE {condition} order by {order_by};'.format(
-                module=module,
-                condition=condition,
-                order_by=order_by,
-            )
-        else:
-            query = 'SELECT * FROM {module} WHERE {condition};'.format(
-                module=module,
-                condition=condition,
-            )
+                # prepare query using search_module_object
 
-        if rule:
-            query = json_logic_replace_data(rule, conf, string_data=query)
+                order_by = ''
+                if search_module_object.get('sort', ''):
+                    order_by = search_module_object.get('sort').get('column') + " " + search_module_object.get(
+                        'sort').get(
+                        'type', "")
 
-        url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
-            sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
+                if limit and order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                        limit=limit)
+                elif limit:
+                    query = 'SELECT * FROM {module} WHERE {condition} LIMIT {limit};'.format(
+                        module=module,
+                        condition=condition,
+                        limit=limit)
+                elif order_by:
+                    query = 'SELECT * FROM {module} WHERE {condition} order by {order_by};'.format(
+                        module=module,
+                        condition=condition,
+                        order_by=order_by,
+                    )
+                else:
+                    query = 'SELECT * FROM {module} WHERE {condition};'.format(
+                        module=module,
+                        condition=condition,
+                    )
 
-        print("URL {}".format(url))
+                if rule:
+                    query = json_logic_replace_data(rule, conf, string_data=query)
 
-        headers = {'content-type': 'application/json'}
-        request_type = 'GET'
-        response, status = invoke_http_request(url, request_type, headers)
+                url = '{vtiger_url}/webservice.php?operation=query&sessionName={sessionName}&query={query}'.format(
+                    sessionName=session_name, query=query, vtiger_url=vtiger_access.get('vtiger_url'))
 
-        print("Query Response: ")
+                print("URL {}".format(url))
+
+                headers = {'content-type': 'application/json'}
+                request_type = 'GET'
+                response, status = invoke_http_request(url, request_type, headers)
+
+                print("Query Response: ")
+
+                if response:
+                    print(response)
+                    if response.get('result') and set_value_fields:
+                        set_value_response = trigger_set_value_task(set_value_fields, conf, rule, session_name,
+                                                                    vtiger_access)
+                        return set_value_response
+
+            elif set_value_fields:
 
-        if response:
-            print(response)
-            if response.get('result') and set_value_fields:
                 set_value_response = trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_access)
                 return set_value_response
-
-    elif set_value_fields:
-
-        set_value_response = trigger_set_value_task(set_value_fields, conf, rule, session_name, vtiger_access)
-        return set_value_response
+        else:
+            print("set_value_configs not found for task id:", task_id)
+    else:
+        print("set_value_configs not found in conf")
 
 
-def invoke_web_service_task(conf, web_service_configs):
+def invoke_web_service_task(conf, task_id):
     """ Web service task:
         params:
         1. conf: conf object
-        2. web_service_configs: json object containing web service task configurations
+        2. task_id: current task id
 
         Execution:
         1. get request object and rule from conf
         2. replace data using json_logic
         3. get request parameters and invoke http request"""
+    if conf.get('web_service_configs', ''):
 
-    rule = web_service_configs.get('rule', '')
-    request_object = web_service_configs.get('request_object', '')
+        if task_id in conf.get('web_service_configs', ''):
+            web_service_configs = conf.get('web_service_configs').get(task_id, '')
 
-    if request_object:
+        if web_service_configs:
 
-        if rule:
-            request_object = json_logic_replace_data(rule, conf, json_data=request_object)
-        url = request_object.get("url")
-        headers = request_object.get("header")
-        request_type = request_object.get("type")
-        payload = request_object.get("payload")
+            rule = web_service_configs.get('rule', '')
+            request_object = web_service_configs.get('request_object', '')
 
-        if payload:
-            if headers.get('content-type', '') == 'application/json':
-                response, status = invoke_http_request(url, request_type, headers, json_data=payload)
-            else:
-                response, status = invoke_http_request(url, request_type, headers, payload=payload)
+            if request_object:
+
+                if rule:
+                    request_object = json_logic_replace_data(rule, conf, json_data=request_object)
+                url = request_object.get("url")
+                headers = request_object.get("header")
+                request_type = request_object.get("type")
+                payload = request_object.get("payload")
+
+                if payload:
+                    if headers.get('content-type', '') == 'application/json':
+                        response, status = invoke_http_request(url, request_type, headers, json_data=payload)
+                    else:
+                        response, status = invoke_http_request(url, request_type, headers, payload=payload)
+                else:
+                    response, status = invoke_http_request(url, request_type, headers)
+                if is_success_request(status):
+                    return response
         else:
-            response, status = invoke_http_request(url, request_type, headers)
-        if is_success_request(status):
-            return response
+            print("web_service_configs not found for task id:", task_id)
+    else:
+        print("web_service_configs not found in conf")
 
 
-def invoke_conditional_task(conf, condition, true_task, false_task):
+def invoke_provider_manager_task(conf, task_id):
+    """ Provider Manager task:
+        params:
+        1. conf: conf object
+        2. task_id: current task id
+
+        Execution:
+        1. get request object and rule from conf
+        2. replace data using json_logic
+        3. get request parameters and invoke http request"""
+    if conf.get('provider_manager_config', ''):
+
+        if task_id in conf.get('provider_manager_config', ''):
+            provider_manager_config = conf.get('provider_manager_config').get(task_id, '')
+
+        if provider_manager_config:
+
+            rule = provider_manager_config.get('rule', '')
+            request_object = provider_manager_config.get('request_object', '')
+
+            if request_object:
+
+                if rule:
+                    request_object = json_logic_replace_data(rule, conf, json_data=request_object)
+                url = request_object.get("url")
+                headers = request_object.get("header")
+                request_type = request_object.get("type")
+                payload = request_object.get("payload")
+
+                if payload:
+                    if headers.get('content-type', '') == 'application/json':
+                        response, status = invoke_http_request(url, request_type, headers, json_data=payload)
+                    else:
+                        response, status = invoke_http_request(url, request_type, headers, payload=payload)
+                else:
+                    response, status = invoke_http_request(url, request_type, headers)
+                if is_success_request(status):
+                    return response
+        else:
+            print("provider_manager_config not found for task id:", task_id)
+    else:
+        print("provider_manager_config not found in conf")
+
+
+def invoke_conditional_task(conf, task_id, true_task, false_task):
     """ this will execute conditional task.
         params:
         1. conf: conf object
-        2. condition: condition object
+        2. task_id: current task id
         3. true_task: task to be triggered if condition is true
         4. false_task: task to be triggered if condition is false
         """
-    is_valid = jsonLogic(condition, conf, ops)
+    if conf.get('conditions', ''):
 
-    if is_valid:
-        return true_task
+        if task_id in conf.get('conditions', ''):
+            condition = conf.get('conditions').get(task_id, '')
+
+        if condition:
+
+            is_valid = jsonLogic(condition, conf, ops)
+
+            if is_valid:
+                return true_task
+            else:
+                return false_task
+        else:
+            print("conditions not found for task id:", task_id)
     else:
-        return false_task
+        print("conditions not found in conf")
 
 
 def invoke_start_task(conf, condition=None, true_task=None, false_task=None, default_task=None):
@@ -571,22 +663,34 @@ def invoke_start_task(conf, condition=None, true_task=None, false_task=None, def
     if conf.get('next_task', ''):
         return conf.get('next_task')
     elif condition and true_task and false_task:
-        task = invoke_conditional_task(conf, condition, true_task, false_task)
-        return task
+
+        is_valid = jsonLogic(condition, conf, ops)
+
+        if is_valid:
+            return true_task
+        else:
+            return false_task
+
     elif default_task:
         return default_task
 
 
-def add_delay_task(conf, current_task_id, vtiger_access):
+def add_delay_task(conf, task_id, vtiger_access):
+    print("task_id", task_id)
     """ This task is useful to insert delayed task data in MySQL database.
             params:
             1. conf: conf object
-            2. current_task_id: current task id
-            3. vtiger_access: vtiger_access to pass configurations
+            2. task_id: current task id
+            3. vtiger_access : json object containing vtiger credentials
             """
     delay_task_configs = conf['delay_task_configs']
-    if current_task_id in delay_task_configs:
-        delay_task_configs = delay_task_configs[current_task_id]
+    eventtype = conf.get('eventtype', '')
+    if eventtype:
+        if eventtype == 'import' or eventtype == 'manual':
+            # send list of payload if event type is import or manual
+            conf['data'] = [conf['data']]
+    if task_id in delay_task_configs:
+        delay_task_configs = delay_task_configs[task_id]
         print("delay_task_configs : ", delay_task_configs)
         payload = {
             "conf": conf,
